@@ -19,6 +19,12 @@
 #include <stdbool.h>
 #include <stdio.h>
 
+#ifdef HAVE_SYS_UTSNAME_H
+
+#include <sys/utsname.h>
+
+#endif
+
 #ifdef HAVE_LIBREADLINE
 
 #if defined(HAVE_READLINE_READLINE_H)
@@ -113,6 +119,9 @@ static struct option long_options[] =
 	{"on-exit-clean", no_argument, 0, 53},
 	{"on-exit-reset", no_argument, 0, 54},
 	{"on-exit-erase-line", no_argument, 0, 55},
+	{"info", no_argument, 0, 56},
+	{"on-exit-sgr0", no_argument, 0, 57},
+	{"direct-color", no_argument, 0, 58},
 	{0, 0, 0, 0}
 };
 
@@ -227,20 +236,37 @@ buildargv(const char *input, int *_argc, char *appname)
 }
 
 static void
-print_versions(void)
+print_version(void)
 {
 	fprintf(stdout, "pspg-%s\n", PSPG_VERSION);
+}
+
+static void
+print_info(void)
+{
+
+	struct utsname u_name;
+
+	fprintf(stdout, "pspg-%s\n", PSPG_VERSION);
 
 #ifdef HAVE_LIBREADLINE
 
 	fprintf(stdout, "with readline (version: 0x%04x)\n", RL_READLINE_VERSION);
 
+#else
+
+	fprintf(stdout, "without readline\n");
+
 #endif
 
 #ifdef COMPILE_MENU
 
 	fprintf(stdout, "with integrated menu\n");
 
+#else
+
+	fprintf(stdout, "without integrated menu\n");
+
 #endif
 
 #ifdef NCURSES_VERSION
@@ -251,10 +277,15 @@ print_versions(void)
 
 #endif
 
+
 #ifdef HAVE_NCURSESW
 
 	fprintf(stdout, "ncurses with wide char support\n");
 
+#else
+
+	fprintf(stdout, "without wide char support\n");
+
 #endif
 
 #ifdef NCURSES_WIDECHAR
@@ -265,20 +296,69 @@ print_versions(void)
 
 	fprintf(stdout, "wchar_t width: %d, max: %d\n", __SIZEOF_WCHAR_T__, __WCHAR_MAX__);
 
+#if NCURSES_EXT_FUNCS
+
+	fprintf(stdout, "with ncurses extended functions support no: %d\n", NCURSES_EXT_FUNCS);
+
+#else
+
+	fprintf(stdout, "without ncurses extended function support\n")
+
+#endif
+
+#ifdef NCURSES_EXT_COLORS
+
+	fprintf(stdout, "with ncurses extended colors no: %d\n", NCURSES_EXT_COLORS);
+
+#else
+
+	fprintf(stdout, "without ncurses extended colors\n");
+
+#endif
+
+
 #ifdef HAVE_POSTGRESQL
 
 	fprintf(stdout, "with postgres client integration\n");
 
+#else
+
+	fprintf(stdout, "without postgres client\n");
+
 #endif
 
 #if defined(HAVE_INOTIFY)
 
 	fprintf(stdout, "with inotify support\n");
 
-#elif defined(HAVE_KQUEUE)
+#else
+
+	fprintf(stdout, "without inotify support\n");
+
+#endif
+
+
+#if defined(HAVE_KQUEUE)
 
 	fprintf(stdout, "with kqueue support\n");
 
+#else
+
+	fprintf(stdout, "without kqueue support\n");
+
+#endif
+
+#ifdef HAVE_SYS_UTSNAME_H
+
+	if (uname(&u_name) != -1)
+	{
+		fprintf(stdout, "%s %s %s %s %s\n", u_name.sysname,
+										 u_name.nodename,
+										 u_name.release,
+										 u_name.version,
+										 u_name.machine);
+	}
+
 #endif
 
 }
@@ -313,6 +393,8 @@ readargs(char **argv,
 					fprintf(stdout, "  --about                  about authors\n");
 					fprintf(stdout, "  --help                   show this help\n");
 					fprintf(stdout, "  -V, --version            show version\n");
+					fprintf(stdout, "  --info                   show info about libraries and system\n");
+					fprintf(stdout, "  --direct-color           use direct true colors\n");
 					fprintf(stdout, "  -f, --file=FILE          open file\n");
 					fprintf(stdout, "  -F, --quit-if-one-screen\n");
 					fprintf(stdout, "                           quit if content is one screen\n");
@@ -378,7 +460,8 @@ readargs(char **argv,
 					fprintf(stdout, "\nOn exit options:\n");
 					fprintf(stdout, "  --on-exit-reset          sends reset terminal sequence \"\\33c\"\n");
 					fprintf(stdout, "  --on-exit-clean          sends clean terminal sequence \"\\033[2J\"\n");
-					fprintf(stdout, "  --on-exit-erase-line     sends erase line terminal sequence \"\\33[2K\\r\"\n");
+					fprintf(stdout, "  --on-exit-erase-line     sends erase line terminal sequence \"\\033[2K\\r\"\n");
+					fprintf(stdout, "  --on-exit-sgr0           sends sgr0 terminal sequence \"\\033[0;10m\"\n");
 					fprintf(stdout, "\nWatch mode options:\n");
 					fprintf(stdout, "  -q, --query=QUERY        execute query\n");
 					fprintf(stdout, "  -w, --watch time         the query (or read file) is repeated every time (sec)\n");
@@ -463,7 +546,7 @@ readargs(char **argv,
 				{
 					fprintf(stdout, "The pspg-%s is special pager designed for databases.\n\n", PSPG_VERSION);
 					fprintf(stdout, "Authors:\n");
-					fprintf(stdout, "    2017-2021 Pavel Stehule, Benesov district, Czech Republic\n\n");
+					fprintf(stdout, "    2017-2022 Pavel Stehule, Benesov district, Czech Republic\n\n");
 					fprintf(stdout, "Licence:\n");
 					fprintf(stdout, "    Distributed under BSD licence\n\n");
 
@@ -587,7 +670,7 @@ readargs(char **argv,
 				}
 				break;
 			case 'V':
-				print_versions();
+				print_version();
 				return false;
 			case 'X':
 				state->no_alternate_screen = true;
@@ -709,6 +792,31 @@ readargs(char **argv,
 				opts->on_exit_erase_line = true;
 				break;
 
+			case 56:
+				print_info();
+				return false;
+
+			case 57:
+				opts->on_exit_sgr0 = true;
+				break;
+
+			case 58:
+
+#ifdef NCURSES_EXT_COLORS
+
+				opts->direct_color = true;
+				break;
+
+#else
+
+				state->errstr = "direct color mode requires ncurses with extended function support";
+				return false;
+
+#endif
+
+				break;
+
+
 			default:
 				{
 					format_error("Try %s --help\n", argv[0]);
