@@ -41,26 +41,29 @@ typedef struct
 	unsigned int rgb;
 } ColorCacheItem;
 
-static ColorCacheItem ColorCache[255];
-static int		nColorCache;
+#define COLOR_CACHE_SIZE		255
 
-const PspgColor PspgBlack = {PSPG_COLOR_BASIC, PSPG_BLACK_COLOR, 0};
-const PspgColor PspgRed = {PSPG_COLOR_BASIC, PSPG_RED_COLOR, 0};
-const PspgColor PspgGreen = {PSPG_COLOR_BASIC, PSPG_GREEN_COLOR, 0};
-const PspgColor PspgBrown = {PSPG_COLOR_BASIC, PSPG_BROWN_COLOR, 0};
-const PspgColor PspgBlue = {PSPG_COLOR_BASIC, PSPG_BLUE_COLOR, 0};
-const PspgColor PspgMagenta = {PSPG_COLOR_BASIC, PSPG_MAGENTA_COLOR, 0};
-const PspgColor PspgCyan = {PSPG_COLOR_BASIC, PSPG_CYAN_COLOR, 0};
-const PspgColor PspgLightGray = {PSPG_COLOR_BASIC, PSPG_LIGHT_GRAY_COLOR, 0};
-const PspgColor PspgGray = {PSPG_COLOR_BASIC, PSPG_GRAY_COLOR, 0};
-const PspgColor PspgBrightRed = {PSPG_COLOR_BASIC, PSPG_BRIGHT_RED_COLOR, 0};
-const PspgColor PspgBrightGreen = {PSPG_COLOR_BASIC, PSPG_BRIGHT_GREEN_COLOR, 0};
-const PspgColor PspgYellow = {PSPG_COLOR_BASIC, PSPG_YELLOW_COLOR, 0};
-const PspgColor PspgBrightBlue = {PSPG_COLOR_BASIC, PSPG_BRIGHT_BLUE_COLOR, 0};
-const PspgColor PspgBrightMagenta = {PSPG_COLOR_BASIC, PSPG_BRIGHT_MAGENTA_COLOR, 0};
-const PspgColor PspgBrightCyan = {PSPG_COLOR_BASIC, PSPG_BRIGHT_CYAN_COLOR, 0};
-const PspgColor PspgWhite = {PSPG_COLOR_BASIC, PSPG_WHITE_COLOR, 0};
-const PspgColor PspgDefault = {PSPG_COLOR_BASIC, PSPG_DEFAULT_COLOR, 0};
+static ColorCacheItem ColorCache[COLOR_CACHE_SIZE];
+static int		nColorCache;
+static bool		use_direct_color;
+
+const PspgColor PspgBlack = {PSPG_COLOR_BASIC, PSPG_BLACK_COLOR, 0x000000};
+const PspgColor PspgRed = {PSPG_COLOR_BASIC, PSPG_RED_COLOR, 0xaa0000};
+const PspgColor PspgGreen = {PSPG_COLOR_BASIC, PSPG_GREEN_COLOR, 0x00aa00};
+const PspgColor PspgBrown = {PSPG_COLOR_BASIC, PSPG_BROWN_COLOR, 0xaa5500};
+const PspgColor PspgBlue = {PSPG_COLOR_BASIC, PSPG_BLUE_COLOR, 0x0000aa};
+const PspgColor PspgMagenta = {PSPG_COLOR_BASIC, PSPG_MAGENTA_COLOR, 0xaa00aa};
+const PspgColor PspgCyan = {PSPG_COLOR_BASIC, PSPG_CYAN_COLOR, 0x00aaaa};
+const PspgColor PspgLightGray = {PSPG_COLOR_BASIC, PSPG_LIGHT_GRAY_COLOR, 0xaaaaaa};
+const PspgColor PspgGray = {PSPG_COLOR_BASIC, PSPG_GRAY_COLOR, 0x555555};
+const PspgColor PspgBrightRed = {PSPG_COLOR_BASIC, PSPG_BRIGHT_RED_COLOR, 0xff5555};
+const PspgColor PspgBrightGreen = {PSPG_COLOR_BASIC, PSPG_BRIGHT_GREEN_COLOR, 0x55ff55};
+const PspgColor PspgYellow = {PSPG_COLOR_BASIC, PSPG_YELLOW_COLOR, 0xffff55};
+const PspgColor PspgBrightBlue = {PSPG_COLOR_BASIC, PSPG_BRIGHT_BLUE_COLOR, 0x5555ff};
+const PspgColor PspgBrightMagenta = {PSPG_COLOR_BASIC, PSPG_BRIGHT_MAGENTA_COLOR, 0xff55ff};
+const PspgColor PspgBrightCyan = {PSPG_COLOR_BASIC, PSPG_BRIGHT_CYAN_COLOR, 0x55ffff};
+const PspgColor PspgWhite = {PSPG_COLOR_BASIC, PSPG_WHITE_COLOR, 0xffffff};
+const PspgColor PspgDefault = {PSPG_COLOR_BASIC, PSPG_DEFAULT_COLOR, -1};
 
 static PspgThemeElement themedef[THEMEDEF_SIZE][2];
 
@@ -89,33 +92,144 @@ styledep(PspgThemeElements el)
 	}
 }
 
+/*
+ * This is taken from tmux
+ */
+static int
+colour_dist_sq(int R, int G, int B, int r, int g, int b)
+{
+	return ((R - r) * (R - r) + (G - g) * (G - g) + (B - b) * (B - b));
+}
+
+static int
+colour_to_6cube(int v)
+{
+	if (v < 48)
+		return (0);
+
+	if (v < 114)
+		return (1);
+
+	return ((v - 35) / 40);
+}
+
+/*
+ * Convert an RGB triplet to the xterm(1) 256 colour palette.
+ *
+ * xterm provides a 6x6x6 colour cube (16 - 231) and 24 greys (232 - 255). We
+ * map our RGB colour to the closest in the cube, also work out the closest
+ * grey, and use the nearest of the two.
+ *
+ * Note that the xterm has much lower resolution for darker colours (they are
+ * not evenly spread out), so our 6 levels are not evenly spread: 0x0, 0x5f
+ * (95), 0x87 (135), 0xaf (175), 0xd7 (215) and 0xff (255). Greys are more
+ * evenly spread (8, 18, 28 ... 238).
+ */
+static int
+colour_find_rgb(unsigned char r,
+				unsigned char g,
+				unsigned char b)
+{
+	static const int	q2c[6] = { 0x00, 0x5f, 0x87, 0xaf, 0xd7, 0xff };
+	int			qr, qg, qb, cr, cg, cb, d, idx;
+	int			grey_avg, grey_idx, grey;
+
+	/* Map RGB to 6x6x6 cube. */
+	qr = colour_to_6cube(r); cr = q2c[qr];
+	qg = colour_to_6cube(g); cg = q2c[qg];
+	qb = colour_to_6cube(b); cb = q2c[qb];
+
+	/* If we have hit the colour exactly, return early. */
+	if (cr == r && cg == g && cb == b)
+		return 16 + (36 * qr) + (6 * qg) + qb;
+
+	/* Work out the closest grey (average of RGB). */
+	grey_avg = (r + g + b) / 3;
+	if (grey_avg > 238)
+		grey_idx = 23;
+	else
+		grey_idx = (grey_avg - 3) / 10;
+	grey = 8 + (10 * grey_idx);
+
+	/* Is grey or 6x6x6 colour closest? */
+	d = colour_dist_sq(cr, cg, cb, r, g, b);
+	if (colour_dist_sq(grey, grey, grey, r, g, b) < d)
+		idx = 232 + grey_idx;
+	else
+		idx = 16 + (36 * qr) + (6 * qg) + qb;
+
+	return idx;
+}
+
 static short
 color_index_rgb(unsigned int rgb)
 {
-	short		r;
-	short		g;
-	short		b;
-
 	int			i;
+	bool		change_color = true;
 
+	/* Use already registered colors */
 	for (i = 0; i <nColorCache; i++)
 	{
 		if (ColorCache[i].rgb == rgb)
 			return ColorCache[i].color;
 	}
 
-	/* rgb is not in cache, new registration is necessary */
-	if (ncurses_color_index >= 255)
+	/*
+	 * When color is not in cache, try to register new color,
+	 * but only when it is allowed, and when the number of color
+	 * is less than COLORS constant.
+	 */
+	if (!can_change_color())
+	{
+		log_row("the terminal doesn't allow to change of color");
+
+		if (COLORS < 256)
+		{
+			log_row("there are not enough colors, use default");
+			return -1;
+		}
+		else
+			change_color = false;
+	}
+
+	if (change_color && ncurses_color_index >= COLORS)
+	{
+		log_row("cannot to define own color, reach limit %d colors", COLORS);
 		return -1;
+	}
 
-	ColorCache[nColorCache].color = ncurses_color_index++;
-	ColorCache[nColorCache].rgb = rgb;
+	if (nColorCache >= COLOR_CACHE_SIZE)
+	{
+		log_row("cannot to define own color, reach limit %d colors in color's cache", COLOR_CACHE_SIZE);
+		return -1;
+	}
 
-	r = ((rgb >> 16) & 0xff) / 255.0 * 1000.0;
-	g = ((rgb >> 8) & 0xff) / 255.0 * 1000.0;
-	b = ((rgb) & 0xff) / 255.0 * 1000.0;
+	if (change_color)
+	{
+		short		r, g, b;
 
-	init_color(ColorCache[nColorCache].color, r, g, b);
+		ColorCache[nColorCache].color = ncurses_color_index++;
+		ColorCache[nColorCache].rgb = rgb;
+
+		r = ((rgb >> 16) & 0xff) / 255.0 * 1000.0;
+		g = ((rgb >> 8) & 0xff) / 255.0 * 1000.0;
+		b = ((rgb) & 0xff) / 255.0 * 1000.0;
+
+		if (init_color(ColorCache[nColorCache].color, r, g, b) == ERR)
+			log_row("init_color function returns ERR");
+	}
+	else
+	{
+		unsigned char		r, g, b;
+
+		/* try to find most near color in 256 color palete */
+		r = ((rgb >> 16) & 0xff);
+		g = ((rgb >> 8) & 0xff);
+		b = ((rgb) & 0xff);
+
+		ColorCache[nColorCache].rgb = rgb;
+		ColorCache[nColorCache].color = colour_find_rgb(r, g, b);
+	}
 
 	return ColorCache[nColorCache++].color;
 }
@@ -142,7 +256,7 @@ deftheme_rgb(PspgThemeElements idx, unsigned int fg, unsigned int bg, int attr)
 	te->attr = attr;
 }
 
-int
+static int
 ncurses_color(PspgBasicColor cv, bool *isbright)
 {
 	*isbright = false;
@@ -196,11 +310,13 @@ ncurses_color(PspgBasicColor cv, bool *isbright)
 	return -1;
 }
 
-int
+static int
 ncurses_color_ex(PspgColor c)
 {
 	if (c.cp == PSPG_COLOR_RGB)
+	{
 		return color_index_rgb(c.rgb);
+	}
 	else if (c.cp == PSPG_COLOR_BASIC)
 	{
 		bool	isbright;
@@ -211,7 +327,7 @@ ncurses_color_ex(PspgColor c)
 		return isbright ? result + 8 : result;
 	}
 	else
-		return 0;
+		return c.bc;
 }
 
 attr_t
@@ -244,7 +360,12 @@ ncurses_theme_attr(PspgThemeElements idx)
 		}
 	}
 
-	if (te->fg.cp == PSPG_COLOR_BASIC &&
+	if (use_direct_color)
+	{
+		fgcolor = te->fg.rgb;
+		bgcolor = te->bg.rgb;
+	}
+	else if (te->fg.cp == PSPG_COLOR_BASIC &&
 		te->bg.cp == PSPG_COLOR_BASIC)
 	{
 		fgcolor = ncurses_color(te->fg.bc, &fgcolorbright);
@@ -290,11 +411,13 @@ ncurses_theme_attr(PspgThemeElements idx)
 	else
 	{
 		/* fallback */
-		fgcolor = COLOR_WHITE;
-		bgcolor = COLOR_BLACK;
+		fgcolor = -1;
+		bgcolor = -1;
 	}
 
-	/* try to find color pair in cache */
+	/* try to find color pair in cache - The number of color pairs can be
+	 * limmited, so try to reuse it.
+	 */
 	for (i = 0; i < nColorPairCache; i++)
 	{
 		if (ColorPairCache[i].fg == fgcolor &&
@@ -312,7 +435,29 @@ ncurses_theme_attr(PspgThemeElements idx)
 	 * The number of color pairs can be limmited, so try
 	 * to reuse it.
 	 */
-	init_pair(ncurses_colorpair_index, fgcolor, bgcolor);
+	if (use_direct_color)
+	{
+
+#ifdef  NCURSES_EXT_COLORS
+
+		if (init_extended_pair(ncurses_colorpair_index, fgcolor, bgcolor) == ERR)
+			log_row("the function init_extended_color_pair returns ERR");
+
+#else
+
+		/* fallback */
+		init_pair(ncurses_colorpair_index, -1, -1);
+		log_row("init_extended_pair function is not available");
+
+#endif
+
+	}
+	else
+	{
+		if (init_pair(ncurses_colorpair_index, fgcolor, bgcolor) == ERR)
+			log_row("the function init_pair returns ERR");
+	}
+
 	result |= COLOR_PAIR(ncurses_colorpair_index);
 
 	ColorPairCache[nColorPairCache].fg = fgcolor;
@@ -334,7 +479,7 @@ typedef struct
  * Set theme definition
  */
 void
-initialize_color_pairs(int theme)
+initialize_color_pairs(int theme, bool direct_color)
 {
 	int		i;
 
@@ -346,6 +491,14 @@ initialize_color_pairs(int theme)
 
 	has_odd_themedef = false;
 
+	if (direct_color)
+	{
+		use_direct_color = true;
+		log_row("direct color mode");
+	}
+	else
+		use_direct_color = false;
+
 	/* reset colors */
 	start_color();
 	use_default_colors();
